package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.dto.DataPermissionDto;
import com.qmth.sop.business.bean.result.TBCrmResult;
import com.qmth.sop.business.bean.result.UserArchivesAllocationResult;
import com.qmth.sop.business.bean.result.UserArchivesAllocationSubTotalResult;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBService;
import com.qmth.sop.business.mapper.ServiceAnalyseMapper;
import com.qmth.sop.business.service.*;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.CrmStatusEnum;
import com.qmth.sop.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

@Service
public class ServiceAnalyseServiceImpl extends ServiceImpl<ServiceAnalyseMapper, TBService> implements ServiceAnalyseService {

    @Resource
    TBSopInfoService tbSopInfoService;

    @Resource
    TBUserArchivesSupplierService tbUserArchivesSupplierService;

    @Resource
    private SysUserService sysUserService;

    @Resource
    TBUserArchivesAllocationService tbUserArchivesAllocationService;

    @Override
    public List<TBService> list(Long startTime, Long endTime) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionDto dpr = sysUserService.buildUserDataPermission(requestUser.getId());
        return this.baseMapper.list(startTime, endTime, dpr);
    }

    /**
     * 4.服务单元概览
     * ①项目执行进度=服务单元下已完成的派单（研究生派单下SOP已完结；教务处派单下已有完成的SOP，无在执行的SOP。可能存在教务处多个SOP之间空窗期导致数据的误差，这个指标允许少量误差存在）；
     * ②设备出库总量：当前服务单元下已出库且尚未入库的设备总量；
     * ③设备占用率：（设备出库总量÷正常设备总量）×100%；
     * ④在服务人员总数：在执行的SOP现场服务人员数量，包括助理工程师、实施工程师、区域协调人；
     * ⑤现场人员占用率：（在服务人员总数÷认证有效的在服务人员总数）×100%；
     */
    @Override
    public Map<String, Object> overview(Long serviceUnitId) {
        Map<String, Object> map = new HashMap<>();
        List<Map<String, Object>> list = this.baseMapper.projectProgress(serviceUnitId);
        int finishCount = 0, size = 0;
        if (!CollectionUtils.isEmpty(list)) {
            size = list.size();
            finishCount = (int) list.stream().filter(s -> Objects.nonNull(s.get("status")) && s.get("status").equals("FINISH")).count();
        }
        BigDecimal sizeBig = BigDecimal.valueOf(size), percent = BigDecimal.valueOf(100),
                finishCompleteRate = BigDecimal.valueOf(finishCount);
        finishCompleteRate = size > 0 ? finishCompleteRate.divide(sizeBig, 2, BigDecimal.ROUND_HALF_UP).multiply(percent).setScale(2, BigDecimal.ROUND_HALF_UP) : sizeBig;
        map.put("projectProgress", finishCompleteRate);
        List<Map<String, Object>> deviceMap = this.baseMapper.equipmentOutboundTotal(serviceUnitId);
        map.put("equipmentOutboundTotal", !CollectionUtils.isEmpty(deviceMap) ? Integer.parseInt(deviceMap.get(0).get("outs").toString()) : 0);
        int totalDevices = !CollectionUtils.isEmpty(deviceMap) ? Integer.parseInt(deviceMap.get(0).get("deviceSum").toString()) : 0;
        BigDecimal totalDevicesBig = BigDecimal.valueOf(totalDevices),
                OutboundTotal = BigDecimal.valueOf(Integer.parseInt(map.get("equipmentOutboundTotal").toString())),
                deviceRate = totalDevices > 0 ? OutboundTotal.divide(totalDevicesBig, 2, BigDecimal.ROUND_HALF_UP).multiply(percent).setScale(2, BigDecimal.ROUND_HALF_UP) : OutboundTotal;
        map.put("equipmentOccupancyRate", deviceRate);
        List<Map<String, Object>> sopPassageMap = tbSopInfoService.findSopPassage(serviceUnitId);
        Integer distributed = 0;
        if (!CollectionUtils.isEmpty(sopPassageMap)) {
            for (Map m : sopPassageMap) {
                //需要去重
                String regionUserStr = Objects.nonNull(m) && Objects.nonNull(m.get("regionUserIds")) ? m.get("regionUserIds").toString() : null;
                if (Objects.nonNull(regionUserStr)) {
                    String[] strs = regionUserStr.split(",");
                    Set<String> set = new HashSet<>(Arrays.asList(strs));
                    distributed = distributed + set.size();
                }
                String engineerUserStr = Objects.nonNull(m) && Objects.nonNull(m.get("engineerUserIds")) ? m.get("engineerUserIds").toString() : null;
                if (Objects.nonNull(engineerUserStr)) {
                    String[] strs = engineerUserStr.split(",");
                    Set<String> set = new HashSet<>(Arrays.asList(strs));
                    distributed = distributed + set.size();
                }
                String assistantEngineerUserId = Objects.nonNull(m) && Objects.nonNull(m.get("assistantEngineerUserIds")) ? m.get("assistantEngineerUserIds").toString() : null;
                if (Objects.nonNull(assistantEngineerUserId)) {
                    String[] strs = assistantEngineerUserId.split(",");
                    Set<String> set = new HashSet<>(Arrays.asList(strs));
                    distributed = distributed + set.size();
                }
            }
        }
        map.put("servicePersonnelTotal", distributed);
        //认证有效的人员总数
        int totalUserArchives = tbUserArchivesSupplierService.countAuthentication();
        BigDecimal totalUserBig = BigDecimal.valueOf(totalUserArchives),
                userServicesTotal = BigDecimal.valueOf(distributed),
                userRate = totalUserArchives > 0 ? userServicesTotal.divide(totalUserBig, 2, BigDecimal.ROUND_HALF_UP).multiply(percent).setScale(2, BigDecimal.ROUND_HALF_UP) : userServicesTotal;
        map.put("sitePersonnelOccupancyRate", userRate);
        return map;
    }

    /**
     * ①当前服务单元已划分了区域协调人的项目总数、供应商分布占比；
     */
    @Override
    public Map<String, Object> supplier(Long serviceUnitId) {
        Map<String, Object> map = new HashMap<>();
        map.put("projectCount", this.baseMapper.projectCount(serviceUnitId));
        List<Map<String, Object>> maps = this.baseMapper.supplierDistribution(serviceUnitId);
//        maps = maps.stream().peek(m -> m.put("percent",(Integer) map.get("projectCount")==0?0: ((Long) m.get("count")) * 100 / ((Integer) map.get("projectCount")) + "%")).collect(Collectors.toList());
        map.put("supplierDistribution", maps);
        return map;
    }

    @Override
    public IPage<TBCrmResult> supplierDetail(Page<Object> iPage, Long serviceUnitId, Long supplierId) {
        return this.baseMapper.crmDetail(iPage, serviceUnitId, supplierId);
    }

    @Override
    public List<Map<String, Object>> supplierRole(Long serviceUnitId, Long supplierId) {
        return this.baseMapper.supplierRole(serviceUnitId, supplierId);
    }

    @Override
    public List<Map<String, Object>> supplierRegion(Long serviceUnitId, Long supplierId) {
        return this.baseMapper.supplierRegion(serviceUnitId, supplierId);
    }


    @Override
    public List<Map<String, Object>> device(Long serviceUnitId) {
        return this.baseMapper.device(serviceUnitId);
    }

    @Override
    public List<Map<String, Object>> personnel(Long serviceUnitId) {
        return this.baseMapper.personnel(serviceUnitId);
    }


    @Override
    public UserArchivesAllocationSubTotalResult findCrmAllocationSubTotal(Long serviceUnitId, Long regionId, Long supplierId) {

//        List<UserArchivesAllocationResult> list = this.baseMapper.findCrmAllocation(serviceUnitId, regionId, supplierId);
        IPage<UserArchivesAllocationResult> userArchivesAllocationResultIPage = tbUserArchivesAllocationService.findCrmAllocationPage(serviceUnitId, null, null, null, null, null, regionId, SystemConstant.PAGE_NUMBER_MIN, SystemConstant.PAGE_SIZE_MAX_SELECT);
        Integer publishedCrmCount = 0, totalCrmCount = 0, coordinatorQuota = 0, coordinatorDistributed = 0, effectQuota = 0,
                effectDistributed = 0, assistantQuota = 0, assistantDistributed = 0;
        if (Objects.nonNull(userArchivesAllocationResultIPage)) {
            List<UserArchivesAllocationResult> list = userArchivesAllocationResultIPage.getRecords();
            publishedCrmCount = Math.toIntExact(list.stream().filter(e -> CrmStatusEnum.PUBLISH.equals(e.getCrmStatus())).count());
            totalCrmCount = list.size();
            coordinatorQuota = list.stream().mapToInt(UserArchivesAllocationResult::getCoordinatorQuota).sum();
            coordinatorDistributed = list.stream().mapToInt(UserArchivesAllocationResult::getCoordinatorDistributed).sum();
            effectQuota = list.stream().mapToInt(UserArchivesAllocationResult::getEffectQuota).sum();
            effectDistributed = list.stream().mapToInt(UserArchivesAllocationResult::getEffectDistributed).sum();
            assistantQuota = list.stream().mapToInt(UserArchivesAllocationResult::getAssistantQuota).sum();
            assistantDistributed = list.stream().mapToInt(UserArchivesAllocationResult::getAssistantDistributed).sum();
        }
        UserArchivesAllocationSubTotalResult result = new UserArchivesAllocationSubTotalResult();
        result.setPublishedCrmCount(publishedCrmCount);
        result.setTotalCrmCount(totalCrmCount);
        result.setCoordinatorDistributed(coordinatorDistributed);
        result.setCoordinatorQuota(coordinatorQuota);
        result.setEffectDistributed(effectDistributed);
        result.setEffectQuota(effectQuota);
        result.setAssistantDistributed(assistantDistributed);
        result.setAssistantQuota(assistantQuota);
        return result;
    }

    @Override
    public List<UserArchivesAllocationResult> findCrmAllocation(Long serviceUnitId, Long regionId, Long supplierId) {
        return this.baseMapper.findCrmAllocation(serviceUnitId, regionId, supplierId);
    }

    /**
     * 查找当前大区待划定派单数量
     *
     * @param regionId
     * @return
     */
    @Override
    public int findWaitZone(Long regionId) {
        return this.baseMapper.findWaitZone(regionId);
    }
}
